LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE std.env.all;
    
entity bcd2sevenseg_tb IS
end bcd2sevenseg_tb;
 
architecture behavior OF bcd2sevenseg_tb IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
    component bcd2sevenseg 
    port (
          digit_selection_in    : in std_logic;                     -- Digit Selection input
          bcd                   : in std_logic_vector(3 downto 0);  -- BCD input
          seven_segment         : out std_logic_vector(6 downto 0); -- 7-segment output MSBit is g, LSBit is a
          digit_selection_out   : out std_logic                     -- Digit Selection Pin for PmodSSD
        );
    end component;
    
  	--Inputs
	signal bcd                     : std_logic_vector(3 downto 0) := (others => 'X');
	signal digit_selection_in      : std_logic;
  	--Outputs
	signal seven_segment           : std_logic_vector(6 downto 0);
	signal digit_selection_out     : std_logic;
	signal a,b,c,d,e,f,g           : std_logic;

	type response_array is array (0 to 15) of std_logic_vector(seven_segment'range);
	constant response : response_array := (
			"0111111",
			"0000110",
			"1011011",
			"1001111",
			"1100110",
			"1101101",
			"1111101",
			"0000111",
			"1111111",
			"1101111",
			"0000000",
			"0000000",
			"0000000",
			"0000000",
			"0000000",
			"0000000"
		);
begin
	g <= seven_segment(6);
	f <= seven_segment(5);
	e <= seven_segment(4);
	d <= seven_segment(3);
	c <= seven_segment(2);
	b <= seven_segment(1);
	a <= seven_segment(0);

 	-- Instantiate the Unit Under Test (UUT)
   uut: bcd2sevenseg PORT MAP (
            digit_selection_in => digit_selection_in,
			bcd => bcd,
			seven_segment => seven_segment,
			digit_selection_out => digit_selection_out
        );

   -- Stimulus process
	stim_proc: process
	begin		
	wait for 20 ns;
		digit_selection_in <='0';
		for i in 0 to 15 loop   
			bcd <= std_logic_vector(to_unsigned(i, bcd'LENGTH));
			wait for 20 ns;
			assert seven_segment=response(i) report "SEVEN SEGMENT OUTPUT FOR INPUT WITH INTEGER VALUE=" & integer'image(i) & " IS WRONG! " severity FAILURE;
			assert digit_selection_out=digit_selection_in report "SEVEN SEGMENT DIGIT SELECTION OUTPUT IS WRONG"  severity FAILURE;
        end loop;
        wait for 20 ns;
		digit_selection_in <='1';
        for i in 0 to 15 loop   
            bcd <= std_logic_vector(to_unsigned(i, bcd'LENGTH));
            wait for 20 ns;
            assert seven_segment=response(i) report "SEVEN SEGMENT OUTPUT FOR INPUT WITH INTEGER VALUE=" & integer'image(i) & " IS WRONG! " severity FAILURE; 
            assert digit_selection_out=digit_selection_in report "SEVEN SEGMENT DIGIT SELECTION OUTPUT IS WRONG"  severity FAILURE;
        end loop;
        		 
		report "ALL PATTERNS APPLIED";
		report "SIMULATION FINISHED SUCCESFULLY!!";
	
		stop(2);	
		
		wait;
	end process;

end;

